@@ -1,28 +1,28 @@
-process_color <- function(col, alpha=NA, sepia.intensity=0, saturation=1) {
+process_color <- function(col, alpha=NA, sepia_intensity=0, saturation=1, color_vision_deficiency_sim="none") {
 	#if (length(col)>100) browser()
-	
+
 	isFactor <- is.factor(col)
-	
+
 	if (isFactor) {
 		x <- as.integer(col)
 		col <- levels(col)
 	}
-	
+
 	res <- t(col2rgb(col, alpha=TRUE))
-	
+
 	# set alpha values
 	if (!is.na(alpha)) res[res[,4] != 0, 4] <- alpha * 255
 
 	# convert to sepia
-	if (sepia.intensity!=0) {
+	if (sepia_intensity!=0) {
 		conv_matrix <- matrix(c(.393, .769, .189,
 								.349, .686, .168,
 								.272, .534, .131), ncol=3, byrow=FALSE)
-		res[,1:3] <-  (res[,1:3] %*% conv_matrix) * sepia.intensity + res[,1:3] * (1-sepia.intensity)
+		res[,1:3] <-  (res[,1:3] %*% conv_matrix) * sepia_intensity + res[,1:3] * (1-sepia_intensity)
 		res[res>255] <- 255
 		res[res<0] <- 0
-	}	
-	
+	}
+
 	# convert to black&white
 	if (saturation!=1) {
 		res[,1:3] <- (res[,1:3] %*% matrix(c(.299, .587, .114), nrow=3, ncol=3))  * (1-saturation) + res[,1:3] * saturation
@@ -32,11 +32,21 @@ process_color <- function(col, alpha=NA, sepia.intensity=0, saturation=1) {
 	if (all(res[,4]==255)) res <- res[,-4, drop=FALSE]
 
 	new_cols <- do.call("rgb", c(unname(as.data.frame(res)), list(maxColorValue=255)))
-	
+
+	rlang::check_installed("colorspace")
+	# color blind sim
+	sim_colors = switch(color_vision_deficiency_sim,
+		deutan = colorspace::deutan,
+		protan = colorspace::protan,
+		tritan = colorspace::tritan,
+		function(x) x)
+
+	new_cols2 = sim_colors(new_cols)
+
 	if (isFactor) {
-		new_cols[x]
+		new_cols2[x]
 	} else {
-		new_cols
+		new_cols2
 	}
 }
 
@@ -54,14 +64,14 @@ get_light <- function(col) {
 darker <- function(colour, rate, alpha=NA) {
 	col <- col2rgb(colour, TRUE)/255
 	col[1:3] <- col[1:3] * (1-rate)
-	if (is.na(alpha)) alpha <- col[4,] 
+	if (is.na(alpha)) alpha <- col[4,]
 	rgb(col[1, ], col[2, ], col[3, ], alpha)
 }
 
 lighter <- function(colour, rate, alpha=NA) {
 	col <- col2rgb(colour, TRUE)/255
 	col[1:3] <- col[1:3] + (1-col[1:3]) * rate
-	if (is.na(alpha)) alpha <- col[4,] 
+	if (is.na(alpha)) alpha <- col[4,]
 	rgb(col[1, ], col[2, ], col[3, ], alpha)
 }
 
@@ -79,14 +89,14 @@ col2hex <- function(x) {
 palette_type <- function(palette) {
 	k <- length(palette)
 	if (k<4) return("cat")
-	
+
 	m1 <- ceiling(k/2) - 1
 	m2 <- floor(k/2) + 1
-	
+
 	colpal_light <- get_light(palette)
-	
+
 	s <- sign(colpal_light[-1] - colpal_light[-k])
-	
+
 	if (all(s==1) || all(s==-1)) {
 		return("seq")
 	} else if (k>4 && ((all(s[1:m1]==1) && all(s[m2:(k-1)]==-1)) ||
@@ -106,11 +116,11 @@ valid_colors <- function(x) {
 
 # get_alpha_col <- function(colour, alpha=NA) {
 # 	col <- col2rgb(colour, TRUE)/255
-# 	if (is.na(alpha)) alpha <- col[4,] 
+# 	if (is.na(alpha)) alpha <- col[4,]
 # 	new_col <- rgb(col[1, ], col[2, ], col[3, ], alpha)
 # 	new_col
 # }
-# 
+#
 # get_sepia_col <- function(col, intensity=1) {
 # 	conv_matrix <- matrix(c(.393, .769, .189,
 # 							.349, .686, .168,
@@ -120,4 +130,4 @@ valid_colors <- function(x) {
 # 	res[res>255] <- 255
 # 	do.call("rgb", c(unname(as.data.frame(res)), list(maxColorValue=255)))
 # }
-# 
\ No newline at end of file
+#
