@@ -62,17 +62,17 @@
 #'   Journal of Statistical Software, 84(6), 1-39, \doi{10.18637/jss.v084.i06}
 #' @export
 qtm = function(shp = NULL,
-				fill = tm_const(),
-				col = tm_const(),
-				size = tm_const(),
-				shape = tm_const(),
-				lwd = tm_const(),
-				lty = tm_const(),
-				fill_alpha = tm_const(),
-				col_alpha = tm_const(),
-				text = tm_const(),
-				text_col = tm_const(),
-				text_size = tm_const(),
+				fill = tmap::tm_const(),
+				col = tmap::tm_const(),
+				size = tmap::tm_const(),
+				shape = tmap::tm_const(),
+				lwd = tmap::tm_const(),
+				lty = tmap::tm_const(),
+				fill_alpha = tmap::tm_const(),
+				col_alpha = tmap::tm_const(),
+				text = tmap::tm_const(),
+				text_col = tmap::tm_const(),
+				text_size = tmap::tm_const(),
 				by = NULL,
 				scale = NULL,
 				title = NULL,
