@@ -4,13 +4,15 @@
 	list(
 		# mode specific options or default values
 		modes = list(plot = list(name = "Grid",
-								 use_gradient = FALSE),
+								 use_gradient = FALSE,
+								 crs_basemap = 3857,
+								 limit_latitude_3857 = c(-84, 84)),
 								 #crs = "auto"),
 					 view = list(name = "Leaflet",
 					 			use_browser = FALSE,
 					 			use_WebGL = NA,
-					 			legend.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "left", align.v = "top", just.h = "left", just.v = "bottom"),
-					 			#crs_data = list(dimensions = 3857, 4326),
+					 			legend.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
+					 			crs_basemap = list(dimensions = 3857, 4326),
 					 			facet.max = 16,
 					 			#legend.bg.alpha = 0.8,
 					 			#view.legend.position = c("right", "top"),
@@ -47,10 +49,11 @@
 		output.dpi = 300,
 		animation.dpi = 100,
 
+
 		# default visual variable values
-		value.const = list(fill.polygons = "grey85",
-						   fill.symbols = "grey60",
+		value.const = list(fill.symbols = "grey60",
 						   fill.dots = "black",
+						   fill = "grey85",
 						   col.polygons = "grey25",
 						   col.symbols = "grey25",
 						   col.raster = "grey40",
@@ -63,16 +66,14 @@
 						   text = "Abc",
 						   text.labels = "",
 						   fontface = NULL,
-						   shape.symbols = 21,
-						   shape.bubbles = 21,
 						   shape.squares = 22,
 						   shape.dots = 19,
 						   shape.markers = marker_icon(),
-						   size.symbols = 1,
+						   shape = 21,
 						   size.bubbles = 1.3333,
 						   size.squares = 1.3333,
 						   size.dots = .3,
-						   size.text = 1,
+						   size = 1,
 						   fill_alpha = 1,
 						   col_alpha = 1,
 						   bgcol_alpha = 1,
@@ -114,7 +115,7 @@
 			col = "#00000000",
 			bgcol = "#00000000",
 			lty = "blank",
-			lwd = 0,
+			lwd = NA,
 			text = "",
 			fontface = NULL,
 			fill_alpha = 0,
@@ -168,22 +169,22 @@
 		),
 
 		# scales
-		scales.var = list(fill = list(fact = "categorical", num = "intervals", int = "discrete"),
-						  col = list(fact = "categorical", num = "intervals", int = "discrete"),
-						  bgcol = list(fact = "categorical", num = "intervals", int = "discrete"),
-						  lwd = list(fact = "categorical", num = "continuous", int = "discrete"),
-						  lty = list(fact = "categorical", num = "intervals"),
-						  shape = list(fact = "categorical", num = "intervals"),
-						  size = list(fact = "continuous", num = "continuous"),
-						  fill_alpha = list(fact = "categorical", num = "intervals"),
-						  col_alpha = list(fact = "categorical", num = "intervals"),
-						  bgcol_alpha = list(fact = "categorical", num = "intervals"),
-						  area = list(fact = "categorical", num = "continuous"),
+		scales.var = list(fill = list(fact = "categorical", num = "intervals", int = "discrete", datetime = "intervals", date = "intervals"),
+						  col = list(fact = "categorical", num = "intervals", int = "discrete", datetime = "intervals", date = "intervals"),
+						  bgcol = list(fact = "categorical", num = "intervals", int = "discrete", datetime = "intervals", date = "intervals"),
+						  lwd = list(fact = "categorical", num = "continuous", int = "discrete", datetime = "intervals", date = "intervals"),
+						  lty = list(fact = "categorical", num = "intervals", datetime = "intervals", date = "intervals"),
+						  shape = list(fact = "categorical", num = "intervals", datetime = "intervals", date = "intervals"),
+						  size = list(fact = "continuous", num = "continuous", datetime = "continuous", date = "continuous"),
+						  fill_alpha = list(fact = "categorical", num = "intervals", datetime = "continuous", date = "continuous"),
+						  col_alpha = list(fact = "categorical", num = "intervals", datetime = "continuous", date = "continuous"),
+						  bgcol_alpha = list(fact = "categorical", num = "intervals", datetime = "continuous", date = "continuous"),
+						  area = list(fact = "categorical", num = "continuous", datetime = "continuous", date = "continuous"),
 						  xmod = list(fact = "asis", num = "asis"),
 						  ymod = list(fact = "asis", num = "asis"),
 						  angle = list(fact = "asis", num = "asis"),
 						  text = list(fact = "asis", num = "asis"),
-						  fontface = list(fact = "categorical", num = "categorical")),
+						  fontface = list(fact = "categorical", num = "categorical", datetime = "categorical", date = "categorical")),
 
 		scale.misc.args = list(continuous = list(n = c(fill = 5, col = 5, 5), outliers.trunc = c(FALSE, FALSE), trans = "identity", limits = list(fill = NA, col = NA, 0)),
 							   continuous_log = list(n = c(fill = 5, col = 5, 5), outliers.trunc = c(FALSE, FALSE),	limits = list(fill = NA, col = NA, NA)),
@@ -224,11 +225,15 @@
 		asp = NA,
 
 		# background
-		bg.color = NA,
-		outer.bg.color = NA,
+		bg = TRUE,
+		bg.color = "white",
+		outer.bg = FALSE,
+		outer.bg.color = "white",
 
 		# frame
 		frame = TRUE,
+		frame.color = NA,
+		frame.alpha = 1,
 		frame.lwd = 1,
 		frame.r = 2,
 		frame.double_line = FALSE,
@@ -242,8 +247,7 @@
 		meta.auto_margins = c(0.4, 0.4, 0.4, 0.4),
 		between_margin = 0.5,
 		panel.margin = c(xtab = 0.4, 0),
-		component.offset = c(inside = 0.75, INSIDE = 0, outside = 0, OUTSIDE = 0),
-		component.stack_margin = 0,
+
 		grid.mark.height = 2,
 		xylab.height = 1.25,
 		coords.height = 1.25,
@@ -297,8 +301,25 @@
 								  out = tm_pos_out(cell.h = "right", cell.v = "center",
 								  		   pos.h = "left", pos.v = "top",
 								  		   align.h = "left", align.v = "top", just.h = "left", just.v = "top")),
-
+		component.offset = c(inside = 0.75, INSIDE = 0, outside = 0, OUTSIDE = 0),
+		component.stack_margin = c(combined = 0, apart = 0.5),
 		component.autoscale = TRUE,
+		component.resize_as_group = FALSE,
+		component.frame_combine = TRUE,
+		component.stack = "vertical",
+		legend.stack = c(all = "vertical", per_row = "horizontal", per_col = "horizontal", all_row = "vertical", all_col = "horizontal", manual = "vertical"),
+		chart.stack = c(all = "vertical", per_row = "horizontal", per_col = "horizontal", all_row = "vertical", all_col = "horizontal", manual = "vertical"),
+		component.equalize = TRUE,
+
+		component.frame = FALSE,
+		component.frame.color = NA,
+		component.frame.alpha = 1,
+		component.frame.lwd = 1,
+		component.frame.r = 2,
+
+		component.bg = TRUE,
+		component.bg.color = "white",
+		component.bg.alpha = 1,
 
 		# legend
 		legend.show = TRUE,
@@ -309,35 +330,35 @@
 										  align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
 		legend.width = NA,
 		legend.height = NA,
-		legend.stack = c(all = "vertical", per_row = "horizontal", per_col = "horizontal", all_row = "vertical", all_col = "horizontal", manual = "vertical"),
-		legend.group.frame = TRUE,
-		legend.resize_as_group = FALSE,
 		legend.reverse = FALSE,
 		legend.na.show = NA,
-		legend.title.color = NULL,
+		legend.title.color = NA,
 		legend.title.size = 0.9,
 		legend.title.fontface = NULL,
 		legend.title.fontfamily = NULL,
 		legend.title.alpha = 1,
-		legend.xlab.color = NULL,
+		legend.xlab.color = NA,
 		legend.xlab.size = 0.9,
 		legend.xlab.fontface = NULL,
 		legend.xlab.fontfamily = NULL,
 		legend.xlab.alpha = 1,
-		legend.ylab.color = NULL,
+		legend.ylab.color = NA,
 		legend.ylab.size = 0.9,
 		legend.ylab.fontface = NULL,
 		legend.ylab.fontfamily = NULL,
 		legend.ylab.alpha = 1,
-		legend.text.color = NULL,
+		legend.text.color = NA,
 		legend.text.size = 0.7,
 		legend.text.fontface = NULL,
 		legend.text.fontfamily = NULL,
 		legend.text.alpha = 1,
 		legend.frame = TRUE,
+		legend.frame.color = NA,
+		legend.frame.alpha = 1,
 		legend.frame.lwd = 1,
 		legend.frame.r = 2,
-		legend.bg.color = NA,
+		legend.bg = TRUE,
+		legend.bg.color = "white",
 		legend.bg.alpha = 1,
 		legend.only = FALSE,
 		legend.absolute_fontsize = 14,
@@ -383,6 +404,9 @@
 												  margins = c(0.4, 0.4, 0.4, 0.4),
 												  item_text.margin = 0.25),
 
+		#
+		add_legend.position = NULL,
+
 		# charts
 		chart.show = TRUE,
 		chart.plot.axis.x = FALSE,
@@ -406,75 +430,61 @@
 						 donut.max = 10,
 						 heatmap.min = 10,
 						 heatmap.max = 10),
-		chart.stack = c(all = "vertical", per_row = "horizontal", per_col = "horizontal", all_row = "vertical", all_col = "horizontal", manual = "vertical"),
-		chart.group.frame = TRUE,
-		chart.resize_as_group = FALSE,
 		chart.reverse = FALSE,
 		chart.na.show = NA,
-		chart.title.color = NULL,
+		chart.title.color = NA,
 		chart.title.size = 0.9,
 		chart.title.fontface = NULL,
 		chart.title.fontfamily = NULL,
 		chart.title.alpha = NULL,
-		chart.xlab.color = NULL,
+		chart.xlab.color = NA,
 		chart.xlab.size = 0.9,
 		chart.xlab.fontface = NULL,
 		chart.xlab.fontfamily = NULL,
 		chart.xlab.alpha = NULL,
-		chart.ylab.color = NULL,
+		chart.ylab.color = NA,
 		chart.ylab.size = 0.9,
 		chart.ylab.fontface = NULL,
 		chart.ylab.fontfamily = NULL,
 		chart.ylab.alpha = 1,
-		chart.text.color = NULL,
+		chart.text.color = NA,
 		chart.text.size = 0.7,
 		chart.text.fontface = NULL,
 		chart.text.fontfamily = NULL,
 		chart.text.alpha = 1,
 		chart.frame = TRUE,
+		chart.frame.color = NA,
+		chart.frame.alpha = 1,
 		chart.frame.lwd = 1,
 		chart.frame.r = 2,
-		chart.bg.color = NA,
+		chart.bg = TRUE,
+		chart.bg.color = "white",
 		chart.bg.alpha = 1,
 		chart.object.color = "#DDDDDD",
 
 		# components
 		title.size = 1.3,
-		title.color = NULL,
+		title.color = NA,
 		title.fontface = NULL,
 		title.fontfamily = NULL,
 		title.alpha = 1,
-		title.bg.color = NA,
-		title.bg.alpha = 1,
 		title.padding = c(0.25, 0.25, 0.25, 0.25),
-
-		title.frame = NA,
+		title.frame = FALSE,
+		title.frame.color = NA,
+		title.frame.alpha = 1,
 		title.frame.lwd = 1,
 		title.frame.r = 2,
-		title.stack = "vertical",
-		title.position = tm_pos_out(cell.h = "center", cell.v ="top", pos.h = "left", pos.v = "top", align.h = "left", align.v = "top", just.h = "left", just.v = "bottom"),
+		title.position = tm_pos_out(cell.h = "center", cell.v ="top", pos.h = "left", pos.v = "top", align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
 		title.width = NA,
-		title.group.frame = TRUE,
-		title.resize_as_group = FALSE,
-
 		credits.size = .7,
 		credits.color = NA,
 		credits.fontface = NULL,
 		credits.fontfamily = NULL,
 		credits.alpha = 1,
-		credits.bg.color = NA,
-		credits.bg.alpha = 1,
 		credits.padding = c(0.25, 0.25, 0.25, 0.25),
-		credits.frame = NA,
-		credits.frame.lwd = 1,
-		credits.frame.r = 2,
-		credits.stack = "vertical",
-		credits.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
+		credits.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 		credits.width = NA,
 		credits.height = NA,
-		credits.group.frame = TRUE,
-		credits.resize_as_group = FALSE,
-
 		compass.north=0,
 		compass.type="arrow",
 		compass.text.size=.8,
@@ -485,32 +495,42 @@
 		compass.color.dark=NA,
 		compass.color.light=NA,
 		compass.lwd=1,
-		compass.bg.color=NA,
-		compass.bg.alpha=1,
 		compass.margins = c(0.25, 0.25, 0.25, 0.25),
-
+		compass.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 		# standard arguments:
-		compass.stack = "vertical",
-		compass.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
-		compass.frame = NA,
-		compass.frame.lwd = 1,
-		compass.frame.r = 2,
-		compass.group.frame = TRUE,
-		compass.resize_as_group = FALSE,
+		inset.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 
 		logo.height = 3,
 		logo.margins = c(0.2, 0.2, 0.2, 0.2),
 		logo.between_margin = 0.2,
-		logo.stack = "vertical",
-		logo.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
-		logo.frame = NA,
-		logo.frame.lwd = 1,
-		logo.frame.r = 2,
-		logo.group.frame = TRUE,
-		logo.resize_as_group = FALSE,
+		logo.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
+
+		inset_map.height = 7,
+		inset_map.width = 7,
+		inset_map.margins = c(0.2, 0.2, 0.2, 0.2),
+		inset_map.between_margin = 0.2,
+		inset_map.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
+		inset_map.frame = TRUE,
+
+		inset.height = 3,
+		inset.width = 3,
+		inset.margins = c(0.2, 0.2, 0.2, 0.2),
+		inset.between_margin = 0.2,
+		inset.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
+		inset.frame = TRUE,
+		inset.bg = TRUE,
+		inset.bg.color = "#ffffff",
+		inset.bg.alpha = 1,
+
+		inset_grob.height = 7,
+		inset_grob.width = 7,
+
+		inset_gg.height = 7,
+		inset_gg.width = 7,
 
 		scalebar.breaks = NULL,
 		scalebar.width = 12,
+		scalebar.allow_clipping = TRUE,
 		scalebar.text.size = .5,
 		scalebar.text.color = NA,
 		scalebar.text.fontface = NULL,
@@ -518,19 +538,11 @@
 		scalebar.color.dark = "black",
 		scalebar.color.light = "white",
 		scalebar.lwd = 1,
-		scalebar.bg.color = NA,
-		scalebar.bg.alpha = 1,
 		scalebar.size = NULL,
 		scalebar.margins = c(0.01,0.01,0.01,0.01),
 
 		# standard arguments:
-		scalebar.stack = "vertical",
-		scalebar.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
-		scalebar.frame = NA,
-		scalebar.frame.lwd = 1,
-		scalebar.frame.r = 2,
-		scalebar.group.frame = TRUE,
-		scalebar.resize_as_group = FALSE,
+		scalebar.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 
 		grid.show = FALSE,
 		grid.labels.pos = c("left", "bottom"),
@@ -562,7 +574,7 @@
 
 		# standard arguments:
 		# mouse.stack = "vertical",
-		# mouse.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "left", align.v = "top", just.h = "left", just.v = "bottom"),
+		# mouse.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
 		# mouse.frame = FALSE,
 		# mouse.frame.lwd = 1,
 		# mouse.frame.r = 2,
@@ -570,13 +582,11 @@
 		# mouse.resize_as_group = FALSE,
 		#
 
-		mouse_coordinates.stack = "vertical",
-		mouse_coordinates.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
+		mouse_coordinates.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 
 		minimap.server = NA,
 		minimap.toggle = TRUE,
-		minimap.stack = "vertical",
-		minimap.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "bottom"),
+		minimap.position = tm_pos_in(pos.h = "right", pos.v = "bottom", align.h = "right", align.v = "top", just.h = "left", just.v = "top"),
 
 		panel.show = NA, # depends on type: TRUE for facets
 		panel.labels = NA,
@@ -585,28 +595,26 @@
 		panel.label.fontface = NULL,
 		panel.label.fontfamily = NULL,
 		panel.label.alpha = 1,
+		panel.label.bg = TRUE,
 		panel.label.bg.color = "grey80",
+		panel.label.bg.alpha = 1,
 		panel.label.frame = TRUE,
+		panel.label.frame.color = NA,
+		panel.label.frame.alpha = 1,
 		panel.label.frame.lwd = 1,
 		panel.label.frame.r = 2,
 		panel.label.height = 1,
-		panel.label.rot = c(90, 0),
-
-		#
-		bbox = NULL,
-		set_bounds = FALSE,
-		set_view = NA,
-		set_zoom_limits = NA,
+		panel.label.rot = c(90, 0, 270, 0),
 
 		qtm.scalebar = FALSE,
 		qtm.minimap = FALSE,
 		qtm.mouse_coordinates = FALSE,
 
 		earth_boundary = FALSE,
-		earth_boundary.color = NULL,
+		earth_boundary.color = NA,
 		earth_boundary.lwd = 1,
 		earth_datum = "OGC:CRS84",
-		space.color = NULL,
+		space.color = "#ffffff",
 
 		check_and_fix = FALSE,
 
@@ -670,9 +678,9 @@ styles = list(
 						  col = list(seq = "brewer.yl_or_br", div = "brewer.rd_yl_gn", unord = "brewer.set3", ord = "brewer.yl_or_br")),
 		frame.lwd = 1,
 		frame.r = 0,
-		title.position = tm_pos_in(pos.h = "left", pos.v = "top", align.h = "left", align.v = "top", just.h = "left", just.v = "bottom"),
+		title.position = tm_pos_in(pos.h = "left", pos.v = "top", align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
 		title.size = 1.3,
-		legend.position = tm_pos_auto_in(align.h = "left", align.v = "top", just.h = "left", just.v = "bottom"),
+		legend.position = tm_pos_auto_in(align.h = "left", align.v = "top", just.h = "left", just.v = "top"),
 		legend.text.size = 0.7,
 		legend.title.size = 0.9,
 		legend.frame = FALSE,
@@ -749,7 +757,9 @@ styles = list(
 		basemap.server = "Esri.NatGeoWorldMap",
 		basemap.alpha = 1),
 	cobalt = list(bg.color = "#002240",
+				  outer.bg = TRUE,
 				  outer.bg.color = "#002240",
+				  space.color = "#002240",
 				  value.const = list(fill.polygons = "#0088FF",
 				  				   fill = "#FF9D00",
 				  				   col.lines = "#FFEE80",
@@ -768,11 +778,15 @@ styles = list(
 				  values.var = list(fill = list(seq = "brewer.yl_gn", div = "brewer.rd_yl_gn", unord = "brewer.set3", ord = "brewer.yl_gn"),
 				  				  col = list(seq = "brewer.yl_gn", div = "brewer.rd_yl_gn", unord = "brewer.set3", ord = "brewer.yl_gn")),
 				  attr.color = "white",
+				  legend.bg.color = "#002240",
 				  chart.text.color = "white",
 				  chart.title.color = "white",
 				  basemap.server = "CartoDB.DarkMatter",
 				  basemap.alpha = .5),
 	albatross = list(bg.color = "#00007F",
+					 outer.bg = TRUE,
+					 outer.bg.color = "#000000",
+					 space.color = "#000000",
 					 value.const = list(fill.polygons = "#4C4C88",
 					 				   fill = "#BFBFFF",
 					 				   col.lines = "#BFBFFF",
@@ -791,6 +805,7 @@ styles = list(
 					 values.var = list(fill = list(seq = "brewer.yl_or_rd", div = "brewer.rd_yl_gn", unord = "brewer.set3", ord = "brewer.yl_or_rd"),
 					 				  col = list(seq = "brewer.yl_or_rd", div = "brewer.rd_yl_gn", unord = "brewer.set3", ord = "brewer.yl_or_rd")),
 					 attr.color = "#BFBFFF",
+					 legend.bg.color = "#00007F",
 					 basemap.alpha = .5),
 	beaver = list(bg.color = "#FFFFFF",
 				  value.const = list(fill.polygons = "#FFE200",
@@ -815,7 +830,7 @@ styles = list(
 				   text.fontfamily = "serif",
 				   frame = TRUE,
 				   frame.double_line = TRUE,
-				   compass.type = "rose"),
+				   inset.type = "rose"),
 	watercolor = list(value.const = list(fill = "#D95F02",
 										 fill.dots = "red",
 										 col.lines = "red",
